/*
 * Copyright (c) 2003, The Regents of the University of California, through
 * Lawrence Berkeley National Laboratory (subject to receipt of any
 * required approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * The GNU GPL Document can be found at:
 * http://www.gnu.org/copyleft/gpl.html
 *
 * Written By Greg Kurtzer <GMKurtzer@lbl.gov> for the Warewulf Cluster 
 * toolkit project (http://warewulf-cluster.org/).
 *
 */


#include <unistd.h>
#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include "getstats.h"
#include "send.h"
#include "config.h"


// We will stay in daemon mode while this is set
int do_loop = '1';
int exitcode = '0';
unsigned int debug = 0;

static void signal_handler(int i) {
	switch(i) {
		case SIGTERM:
			if ( debug ) {
				printf("SIGTERM recieved, ending loop\n");
			}
			do_loop = 0;
			exitcode = 254;
			break;
		case SIGINT:
			if ( debug ) {
				printf("SIGINT recieved, ending loop\n");
			}
			do_loop = 0;
			exitcode = 254;
			break;
		default:
			if ( debug ) {
				printf("SIG* recieved... What's that for?\n");
			}
			exitcode = 255;
			break;
	};
};

static void usage(char *program) {
        printf("usage: %s [options]\n"
               "\n"
               "[options]\n"
               "   -h <hostname>       hostname of the warewulf master [def=localhost]\n"
               "   -d                  debug and no fork()\n"
               "   -h                  this help\n",
               program);
        exit(exitcode);
}


int main (int argc, char **argv) {
	FILE *fd;
	char packet[PACKETSIZE+1];
	char *cpu_info, *uname_info;
	signed char c;
	int pid, oldpid;
        char buffer[BUFFERSIZE+1];
        char hostname[255];
	// Catch some typical signals when thrown this way
	signal(SIGTERM, signal_handler);
	signal(SIGINT, signal_handler);
	signal(SIGHUP, signal_handler);

	strcpy(hostname, "localhost");

        while ((c = getopt (argc, argv, "h:d")) != EOF) {
                switch (c) {
                        case 'h':
                                strcpy(hostname, optarg);
                                break;
                        case 'd':
                                debug++;
                                break;
                        default:
                                usage(argv[0]);
                                break;
                }
        }

	if ((fd = fopen(PIDFILE, "r")) != NULL) {
		fgets(buffer, BUFFERSIZE, fd);
		oldpid = atoi(buffer);
		fprintf(stderr, "Wulf daemon already running; %s, PID: %d\n", PIDFILE, oldpid);
		fclose(fd);
		exit(1);
	}


	// Send a test packet with nothing in it to make sure we
	// resolve properly.
	udp_send(hostname, "");

	if ( ! debug ) {

		if ( (pid = fork() ) < 0 ) {
	//		perror("fork");
			exit(exitcode);
		}
		if(pid != 0) {
			exit(0);
		}
	
		if ((fd = fopen(PIDFILE, "w")) == NULL) {
			fprintf(stderr, "Couldn't write lock file %s!...\ncontinuing anyway...\n", PIDFILE);
			//perror(PIDFILE);
			// nonfatal error..
			// exit(-1);
		} else {
			pid = getpid();
			fprintf(fd, "%d\n", pid);
			fflush(fd);
			fclose(fd);
		}
	}

	// cpu_info shouldnt change while daemon is running, so we should
	// just read it in once
	cpu_info = get_cpu_info();
	uname_info = get_uname();

	// If we have gotten this far, we may as well fork and exit (soon)
	while (do_loop) {
		// Assembly of the packet
		snprintf(packet, PACKETSIZE, "%s%s%s%s%s%s%s%s",
			cpu_info,
			uname_info,
			get_sysinfo(),
			get_cpu_util(),
			get_mem_stats(),
			get_load_avg(),
			get_net_stats(),
			get_node_status());
		if ( debug )
			printf("<packet sendto=%s>\n%s</packet>\n", hostname, packet);
		// Send the packet!
		udp_send(hostname, packet);
		sleep(REFRESH);
	}
	unlink(PIDFILE);
	if ( debug ) {
		printf("wulfd exiting: %i\n", exitcode);
	}
	return(exitcode);
}
